import logging
from vkmax.client import MaxClient

_logger = logging.getLogger(__name__)

async def resolve_users(
        client: MaxClient,
        user_id: list
    ):

    """ Resolving users via userid"""

    return await client.invoke_method(
        opcode = 32,
        payload = {
            "contactIds":user_id
            }
        )

async def add_to_contacts(client: MaxClient, user_id: int):

    """ Adding user to contacts via userid """

    return await client.invoke_method(
        opcode = 34,
        payload = {
            "contactId":user_id,
            "action":"ADD"
            }
        )

async def ban(client: MaxClient, user_id: int):

    """ Banhammer to user's head. """

    return await client.invoke_method(
        opcode = 34,
        payload = {
            "contactId":user_id,
            "action":"BLOCK"
            }
        )



from vkmax.client import MaxClient
from random import randint

async def create_group(
        client: MaxClient,
        group_name: str,
        participant_ids: list[int]
    ):
    
    """ Creates a group """
    
    return await client.invoke_method(
        opcode=64,
        payload={
              "message": {
                  "cid": randint(1750000000000, 2000000000000),  # TODO: fuck around and find out
                  "attaches": [
                      {
                          "_type": "CONTROL",
                          "event": "new",
                          "chatType": "CHAT",
                          "title": group_name,
                          "userIds": participant_ids
                      }
                  ]
              },
              "notify": True
          }
      )

async def invite_users(
        client: MaxClient,
        group_id: int,
        participant_ids: list[int],
        show_history = True
    ):
    
    """ Invites users into group via userid """
    
    return await client.invoke_method(
        opcode=77,
        payload={
            "chatId": group_id,
            "userIds": participant_ids,
            "showHistory": show_history,
            "operation": "add"
            }
        )

async def remove_users(
        client: MaxClient,
        group_id: int,
        participant_ids: list[int],
        delete_messages = False
    ):
    delete_messages = 0 if delete_messages == False else -1
    
    """ Removes users from group via userid """
    
    return await client.invoke_method(
        opcode=77,
        payload={
            "chatId":group_id,
            "userIds":participant_ids,
            "operation":"remove",
            "cleanMsgPeriod":delete_messages
            }
        )

async def add_admin(
        client: MaxClient,
        group_id: int,
        admin_ids = list[int],
        deleting_messages = False,
        control_participants = False,
        control_admins = False
    ):

    """ Adds an admin to group\nYou need to be a group owner or administrator with appropriate permissions to use this method. """
    
    # minimal admin: 120, minimal admin + deleting msgs: 121, minimal admin + deleting msgs + participants: 123, full_admin: 255, minimal admin + set_admins: 124, minimal admin + set_admins + deleting msgs: 125, minimal admin + set_admins + participants: 254, minimal admin + participants: 250, minimal admin + deletings msgs + participants: 251
    permissions = 120 if deleting_messages == False and control_participants == False and control_admins == False else ...
    permissions = 121 if deleting_messages == True and control_participants == False and control_admins == False else ...
    permissions = 123 if deleting_messages == True and control_participants == True and control_admins == False else ...
    permissions = 124 if deleting_messages == False and control_participants == False and control_admins == True else ...
    permissions = 125 if deleting_messages == True and control_participants == False and control_admins == True else ...
    permissions = 250 if deleting_messages == False and control_participants == True and control_admins == False else ...
    permissions = 251 if deleting_messages == True and control_participants == True and control_admins == False else ...
    permissions = 254 if deleting_messages == False and control_participants == True and control_admins == True else ...
    permissions = 255 if deleting_messages == True and control_participants == True and control_admins == True else ...
    
    return await client.invoke_method(
        opcode=77,
        payload = {
            "chatId":group_id,
            "userIds":admin_ids,
            "type":"ADMIN",
            "operation":"remove",
            "permissions": permissions
            }
    )

async def remove_admin(
        client: MaxClient,
        group_id: int,
        admin_ids = list[int]
    ):

    """ Removes an admin from group\nYou need to be a group owner or administrator with appropriate permissions to use this method. """
    
    return await client.invoke_method(
        opcode=77,
        payload = {
            "chatId":group_id,
            "userIds":admin_ids,
            "type":"ADMIN",
            "operation":"remove"
            }
    )

async def transfer_group_ownership(
        client: MaxClient, 
        group_id: int,
        new_owner_id: int
    ):

    """ Transfers ownership of the group to a new user """

    return await client.invoke_method(
        opcode=55,
        payload = {
            "chatId":group_id,
            "changeOwnerId":new_owner_id
            }
    )

async def change_group_settings(
        client: MaxClient,
        group_id : int,
        all_can_pin_message = False,
        only_owner_can_change_icon_title = True,
        only_admin_can_add_member = True

    ):

    """ Changes basic group settings """

    return await client.invoke_method(
        opcode=55,
        payload = {
            "chatId": group_id,
            "options": {
                "ONLY_OWNER_CAN_CHANGE_ICON_TITLE": only_owner_can_change_icon_title,
                "ALL_CAN_PIN_MESSAGE": all_can_pin_message,
                "ONLY_ADMIN_CAN_ADD_MEMBER": only_admin_can_add_member
                }
            }
    )

async def change_group_profile(
        client: MaxClient,
        group_id: int,
        new_group_name: str,
        new_description = None
    ): 

    """ Just changes a group public profile """

    if new_description != None:
        return await client.invoke_method(
            opcode=55,
            payload = {
                "chatId": group_id,
                "theme": new_group_name,
                }
        )
    
    else:
        return await client.invoke_method(
        opcode=55,
        payload = {
            "chatId": group_id,
            "theme": new_group_name,
            "description": new_description
            }
    )

async def get_group_members(
        client: MaxClient,
        group_id: int,
        marker = 0,
        count = 500
    ):

    """ Gets all the members in specified chat """

    if count > 500:
        raise Exception("Error while getting number of group members. Maximum available count is 500.")

    return await client.invoke_method(
        opcode = 59,
        payload = {
            "type":"MEMBER",
            "marker": marker,
            "chatId": group_id,
            "count": count
            }
    )



from dataclasses import dataclass
from typing import Literal, Any


@dataclass
class MaxPacket:
    # Not used anywhere, but that's going to change soon
    ver: int
    cmd: Literal[0, 1]
    opcode: int
    seq: int
    payload: dict[str, Any]



from vkmax.client import MaxClient
from random import randint

async def resolve_channel_username(
        client: MaxClient, 
        username: str
    ):

    """ Only channel resolving """

    return await client.invoke_method(
        opcode = 89,
        payload = {
            "link": f"https://max.ru/{username}"
            }
    )

async def resolve_channel_id(
        client: MaxClient,
        channel_id: int
    ):

    """"""

    return await client.invoke_method(
        opcode = 48,
        payload = {
            "ver": 11,
            "cmd":0,
            "seq":473,
            "opcode":48,
            "payload": {
                "chatIds": [
                        channel_id
                    ]
                }
            }
    )

async def join_channel(
        client: MaxClient,
        username: str
    ):

    """ Joining a channel and resolving """

    return await client.invoke_method(
        opcode = 57,
        payload = {
            "link": f"https://max.ru/{username}"
        }
    )

async def create_channel(
        client: MaxClient,
        channel_name: str
    ):

    """"""

    return await client.invoke_method(
        opcode = 64,
        payload = {
            "message":
                {
                    "cid": randint(1750000000000, 2000000000000),
                    "attaches":
                        [
                            {
                                "_type": "CONTROL",
                                "event": "new",
                                "title": channel_name,
                                "chatType": "CHANNEL"
                            }
                        ],
                    "text": ""
                }
            }
        )

async def mute_channel(
        client: MaxClient,
        channel_id: int,
        mute = True
    ):

    """ Mutes or unmutes a channel """

    return await client.invoke_method(
        opcode = 22,
        payload = {
            "settings": 
                {
                    "chats":
                        {
                            str(channel_id): 
                                {
                                    "dontDisturbUntil": -1 if mute else 0
                                }
                        }
                }
            }
    )




from vkmax.client import MaxClient

async def change_online_status_visibility(
        client: MaxClient,
        hidden: bool
    ):

    """ Hide or show you last online status """

    return await client.invoke_method(
        opcode = 22,
        payload = {
            "settings":
            {
                "user":
                {
                    "HIDDEN": hidden
                }
            }
        }
    )

async def set_is_findable_by_phone(
        client: MaxClient,
        findable: bool
    ):

    """ Changes your profile privacy settings. You can make your profile findable by phone or not. """
    
    findable = "ALL" if findable else "CONTACTS"
    return await client.invoke_method(
        opcode = 22,
        payload = {
            "settings":
            {
                "user":
                {
                    "SEARCH_BY_PHONE": findable
                }
            }
        }
    )

async def set_calls_privacy(
        client: MaxClient,
        can_be_called: bool
    ):

    """ You can enable or disable calls for everyone. """

    findable = "ALL" if can_be_called else "CONTACTS"
    return await client.invoke_method(
        opcode = 22,
        payload = {
            "settings":
            {
                "user":
                {
                    "INCOMING_CALL": findable
                }
            }
        }
    )

async def invite_privacy(
        client: MaxClient,
        invitable:  bool
    ):

    """ Changes privacy settings\nMakes possible or impossible inviting you to other chats """

    invitable = "ALL" if invitable else "CONTACTS"
    return await client.invoke_method(
        opcode = 22,
        payload = {
            "settings": {
                "user":
                {
                    "CHATS_INVITE": invitable
                }
            }
        }
    )

async def change_profile(
        client: MaxClient,
        first_name = None,
        last_name = None,
        bio = None
    ):

    """ Changes your public profile\nRecommended to use with all available options"""

    return await client.invoke_method(
        opcode = 16,
        payload = {
            "firstName": first_name,
            "lastName": last_name,
            "description": bio}
    )
    



from random import randint
from vkmax.client import MaxClient

async def send_message(
        client: MaxClient,
        chat_id: int,
        text: str,
        notify: bool = True
    ):
    
    """ Sends message to specified chat """

    return await client.invoke_method(
        opcode = 64,
        payload = {
                "chatId": chat_id,
                "message": {
                "text": text,
                "cid": randint(1750000000000, 2000000000000),
                "elements": [],
                "attaches": []
            },
            "notify": notify
        }
    )

async def edit_message(
        client: MaxClient,
        chat_id: int,
        message_id: int,
        text: str
    ):

    """ Edits the specified message """

    return await client.invoke_method(
        opcode = 67,
        payload = {
            "chatId": chat_id,
            "messageId": str(message_id),
            "text": text,
            "elements": [],
            "attachments": []
        }
    )

async def delete_message(
        client: MaxClient,
        chat_id: int,
        message_ids: list,
        delete_for_me: bool = False
    ):

    """ Deletes the specified message """

    return await client.invoke_method(
        opcode = 66,
        payload = {
            "chatId": chat_id,
            "messageIds": message_ids,
            "forMe": delete_for_me
        }
    )

async def pin_message(
        client: MaxClient,
        chat_id: int,
        message_id: int,
        notify = False
    ):

    """ Pins message in the chat """

    return await client.invoke_method(
        opcode = 55,
        payload = {
            "chatId": chat_id,
            "notifyPin": notify,
            "pinMessageId": str(message_id)
            }
    )

async def reply_message(
        client: MaxClient,
        chat_id: int,
        text: str,
        reply_to_message_id: int,
        notify = True
    ):

    """ Replies to message in the chat """
    
    return await client.invoke_method(
        opcode = 64,
        payload = {
            "chatId": chat_id,
            "message": {
                "text": text,
                "cid": randint(1750000000000, 2000000000000),
                "elements": [],
                "link": {
                    "type": "REPLY",
                    "messageId": str(reply_to_message_id)
                    },
                "attaches": []
                },
            "notify": notify
            }
    )




import asyncio
import itertools
import json
import logging
import uuid
from typing import Any, Callable, Optional

import websockets
from websockets.asyncio.client import ClientConnection

from functools import wraps

WS_HOST = "wss://ws-api.oneme.ru/websocket"
RPC_VERSION = 11
APP_VERSION = "25.6.8"

_logger = logging.getLogger(__name__)


def ensure_connected(method: Callable):
    @wraps(method)
    def wrapper(self, *args, **kwargs):
        if self._connection is None:
            raise RuntimeError("WebSocket not connected. Call .connect() first.")
        return method(self, *args, **kwargs)

    return wrapper


class MaxClient:
    def __init__(self):
        self._connection: Optional[ClientConnection] = None
        self._is_logged_in: bool = False
        self._seq = itertools.count(1)
        self._keepalive_task: Optional[asyncio.Task] = None
        self._recv_task: Optional[asyncio.Task] = None
        self._incoming_event_callback = None
        self._pending = {}

    # --- WebSocket connection management ---

    async def connect(self):
        if self._connection:
            raise Exception("Already connected")

        _logger.info(f'Connecting to {WS_HOST}...')
        self._connection = await websockets.connect(WS_HOST)

        self._recv_task = asyncio.create_task(self._recv_loop())
        _logger.info('Connected. Receive task started.')
        return self._connection

    @ensure_connected
    async def disconnect(self):
        await self._stop_keepalive_task()
        self._recv_task.cancel()
        await self._connection.close()

    @ensure_connected
    async def invoke_method(self, opcode: int, payload: dict[str, Any]):
        seq = next(self._seq)

        request = {
            "ver": RPC_VERSION,
            "cmd": 0,
            "seq": seq,
            "opcode": opcode,
            "payload": payload
        }
        _logger.info(f'-> REQUEST: {request}')

        future = asyncio.get_event_loop().create_future()
        self._pending[seq] = future

        await self._connection.send(
            json.dumps(request)
        )

        response = await future
        _logger.info(f'<- RESPONSE: {response}')

        return response

    async def set_callback(self, function):
        if not asyncio.iscoroutinefunction(function):
            raise TypeError('callback must be async')
        self._incoming_event_callback = function

    async def _recv_loop(self):
        try:
            async for packet in self._connection:
                packet = json.loads(packet)
                seq = packet["seq"]
                future = self._pending.pop(seq, None)
                if future:
                    future.set_result(packet)
                else:
                    if self._incoming_event_callback:
                        asyncio.create_task(self._incoming_event_callback(self, packet))
        except asyncio.CancelledError:
            _logger.info(f'receiver cancelled')
            return

    # --- Keepalive system

    @ensure_connected
    async def _send_keepalive_packet(self):
        await self.invoke_method(
            opcode=1,
            payload={"interactive": False}
        )

    @ensure_connected
    async def _keepalive_loop(self):
        _logger.info(f'keepalive task started')
        try:
            while True:
                await self._send_keepalive_packet()
                await asyncio.sleep(30)
        except asyncio.CancelledError:
            _logger.info('keepalive task stopped')
            return

    @ensure_connected
    async def _start_keepalive_task(self):
        if self._keepalive_task:
            raise Exception('Keepalive task already started')

        self._keepalive_task = asyncio.create_task(self._keepalive_loop())
        return

    async def _stop_keepalive_task(self):
        if not self._keepalive_task:
            raise Exception('Keepalive task is not running')

        self._keepalive_task.cancel()
        self._keepalive_task = None
        return

    # --- Authentication ---

    @ensure_connected
    async def _send_hello_packet(self):
        return await self.invoke_method(
            opcode=6,
            payload={
                "userAgent": {
                    "deviceType": "WEB",
                    "locale": "ru_RU",
                    "osVersion": "macOS",
                    "deviceName": "vkmax Python",
                    "headerUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/137.0.0.0 Safari/537.36",
                    "deviceLocale": "ru-RU",
                    "appVersion": APP_VERSION,
                    "screen": "956x1470 2.0x",
                    "timezone": "Asia/Vladivostok"
                },
                "deviceId": str(uuid.uuid4())
            }
        )

    @ensure_connected
    async def send_code(self, phone: str) -> str:
        """:returns: Login token."""
        await self._send_hello_packet()
        start_auth_response = await self.invoke_method(
            opcode=17,
            payload={
                "phone": phone,
                "type": "START_AUTH",
                "language": "ru"
            }
        )
        return start_auth_response["payload"]["token"]

    @ensure_connected
    async def sign_in(self, sms_token: str, sms_code: int):
        """
        Auth token for further login is at ['payload']['tokenAttrs']['LOGIN']['token']
        :param login_token: Must be obtained via `send_code`.
        """
        verification_response = await self.invoke_method(
            opcode=18,
            payload={
                "token": sms_token,
                "verifyCode": str(sms_code),
                "authTokenType": "CHECK_CODE"
            }
        )

        if "error" in verification_response["payload"]:
            raise Exception(verification_response["payload"]["error"])

        _logger.info(f'Successfully logged in as {verification_response["payload"]["profile"]["phone"]}')

        self._is_logged_in = True
        await self._start_keepalive_task()

        return verification_response

    @ensure_connected
    async def login_by_token(self, token: str):
        await self._send_hello_packet()
        _logger.info("using session")
        login_response = await self.invoke_method(
            opcode=19,
            payload={
                "interactive": True,
                "token": token,
                "chatsSync": 0,
                "contactsSync": 0,
                "presenceSync": 0,
                "draftsSync": 0,
                "chatsCount": 40
            }
        )

        if "error" in login_response["payload"]:
            raise Exception(login_response["payload"]["error"])

        _logger.info(f'Successfully logged in as {login_response["payload"]["profile"]["phone"]}')

        self._is_logged_in = True
        await self._start_keepalive_task()

        return login_response
